@@ -1,6 +1,6 @@
 
 /**************************************************************************
- * Copyright (C) 2007-2010 Ruben Pollan Bella <meskio@sindominio.net>     *
+ * Copyright (C) 2007-2011 Ruben Pollan Bella <meskio@sindominio.net>     *
  *                                                                        *
  *  This file is part of TuDu.                                            *
  *                                                                        *
@@ -121,7 +121,7 @@ public:
 	 * u = user
 	 * the same leters in capital are for inverse order
 	 */
-	void sort(string order);
+	void sort(wstring order);
 	bool search(wstring& str);
 	bool searchUp(wstring& str);
 protected:
